@@ -160,6 +160,7 @@ def insert_optional_args(args):
 		'--usepkgonly'           : y_or_n,
 		'--verbose'              : y_or_n,
 		'--verbose-slot-rebuilds': y_or_n,
+		'--with-test-deps'       : y_or_n,
 	}
 
 	short_arg_opts = {
@@ -293,7 +294,7 @@ def parse_opts(tmpcmdline, silent=False):
 	actions = frozenset([
 		"clean", "check-news", "config", "depclean", "help",
 		"info", "list-sets", "metadata", "moo",
-		"prune", "regen",  "search",
+		"prune", "rage-clean", "regen",  "search",
 		"sync",  "unmerge", "version",
 	])
 
@@ -661,6 +662,11 @@ def parse_opts(tmpcmdline, silent=False):
 			"help"     : "verbose slot rebuild output",
 			"choices"  : true_y_or_n
 		},
+		"--with-test-deps": {
+			"help"     : "pull in test deps for packages " + \
+				"matched by arguments",
+			"choices"  : true_y_or_n
+		},
 	}
 
 	parser = ArgumentParser(add_help=False)
@@ -956,6 +962,11 @@ def parse_opts(tmpcmdline, silent=False):
 	else:
 		myoptions.verbose = None
 
+	if myoptions.with_test_deps in true_y:
+		myoptions.with_test_deps = True
+	else:
+		myoptions.with_test_deps = None
+
 	for myopt in options:
 		v = getattr(myoptions, myopt.lstrip("--").replace("-", "_"))
 		if v:
