@@ -1,12 +1,13 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: search.py 14870 2009-11-21 04:06:17Z zmedico $
 
 from __future__ import print_function
 
 import re
 import portage
 from portage import os
+from portage.dbapi.porttree import _parse_uri_map
+from portage.localization import localized_size
 from portage.output import  bold, bold as white, darkgreen, green, red
 from portage.util import writemsg_stdout
 
@@ -38,13 +39,6 @@ class search(object):
 		self.matches = {"pkg" : []}
 		self.mlen = 0
 
-		def fake_portdb():
-			pass
-		self.portdb = fake_portdb
-		for attrib in ("aux_get", "cp_all",
-			"xmatch", "findname", "getFetchMap"):
-			setattr(fake_portdb, attrib, getattr(self, "_"+attrib))
-
 		self._dbs = []
 
 		portdb = root_config.trees["porttree"].dbapi
@@ -76,7 +70,7 @@ class search(object):
 				return db.aux_get(*args, **kwargs)
 			except KeyError:
 				pass
-		raise
+		raise KeyError(args[0])
 
 	def _findname(self, *args, **kwargs):
 		for db in self._dbs:
@@ -158,7 +152,7 @@ class search(object):
 					if not result or cpv == portage.best([cpv, result]):
 						result = cpv
 				else:
-					db_keys = Package.metadata_keys
+					db_keys = list(db._aux_cache_keys)
 					# break out of this loop with highest visible
 					# match, checked in descending order
 					for cpv in reversed(db.match(atom)):
@@ -182,10 +176,10 @@ class search(object):
 		self.packagematches = []
 		if self.searchdesc:
 			self.searchdesc=1
-			self.matches = {"pkg":[], "desc":[],}
+			self.matches = {"pkg":[], "desc":[], "set":[]}
 		else:
 			self.searchdesc=0
-			self.matches = {"pkg":[],}
+			self.matches = {"pkg":[], "set":[]}
 		print("Searching...   ", end=' ')
 
 		regexsearch = False
@@ -199,7 +193,8 @@ class search(object):
 			self.searchre=re.compile(self.searchkey,re.I)
 		else:
 			self.searchre=re.compile(re.escape(self.searchkey), re.I)
-		for package in self.portdb.cp_all():
+
+		for package in self._cp_all():
 			self._spinner_update()
 
 			if match_category:
@@ -209,21 +204,21 @@ class search(object):
 
 			masked=0
 			if self.searchre.search(match_string):
-				if not self.portdb.xmatch("match-visible", package):
+				if not self._xmatch("match-visible", package):
 					masked=1
 				self.matches["pkg"].append([package,masked])
 			elif self.searchdesc: # DESCRIPTION searching
-				full_package = self.portdb.xmatch("bestmatch-visible", package)
+				full_package = self._xmatch("bestmatch-visible", package)
 				if not full_package:
 					#no match found; we don't want to query description
 					full_package = portage.best(
-						self.portdb.xmatch("match-all", package))
+						self._xmatch("match-all", package))
 					if not full_package:
 						continue
 					else:
 						masked=1
 				try:
-					full_desc = self.portdb.aux_get(
+					full_desc = self._aux_get(
 						full_package, ["DESCRIPTION"])[0]
 				except KeyError:
 					print("emerge: search: aux_get() failed, skipping")
@@ -231,16 +226,31 @@ class search(object):
 				if self.searchre.search(full_desc):
 					self.matches["desc"].append([full_package,masked])
 
+		self.sdict = self.setconfig.getSets()
+		for setname in self.sdict:
+			self._spinner_update()
+			if match_category:
+				match_string = setname
+			else:
+				match_string = setname.split("/")[-1]
+			
+			if self.searchre.search(match_string):
+				self.matches["set"].append([setname, False])
+			elif self.searchdesc:
+				if self.searchre.search(
+					self.sdict[setname].getMetadata("DESCRIPTION")):
+					self.matches["set"].append([setname, False])
+			
 		self.mlen=0
 		for mtype in self.matches:
 			self.matches[mtype].sort()
 			self.mlen += len(self.matches[mtype])
 
 	def addCP(self, cp):
-		if not self.portdb.xmatch("match-all", cp):
+		if not self._xmatch("match-all", cp):
 			return
 		masked = 0
-		if not self.portdb.xmatch("bestmatch-visible", cp):
+		if not self._xmatch("bestmatch-visible", cp):
 			masked = 1
 		self.matches["pkg"].append([cp, masked])
 		self.mlen += 1
@@ -253,29 +263,42 @@ class search(object):
 		msg.append("[ Applications found : " + \
 			bold(str(self.mlen)) + " ]\n\n")
 		vardb = self.vartree.dbapi
+		metadata_keys = set(Package.metadata_keys)
+		metadata_keys.update(["DESCRIPTION", "HOMEPAGE", "LICENSE", "SRC_URI"])
+		metadata_keys = tuple(metadata_keys)
 		for mtype in self.matches:
 			for match,masked in self.matches[mtype]:
 				full_package = None
 				if mtype == "pkg":
-					catpack = match
-					full_package = self.portdb.xmatch(
+					full_package = self._xmatch(
 						"bestmatch-visible", match)
 					if not full_package:
 						#no match found; we don't want to query description
 						masked=1
 						full_package = portage.best(
-							self.portdb.xmatch("match-all",match))
+							self._xmatch("match-all",match))
 				elif mtype == "desc":
 					full_package = match
 					match        = portage.cpv_getkey(match)
-
+				elif mtype == "set":
+					msg.append(green("*") + "  " + bold(match) + "\n")
+					if self.verbose:
+						msg.append("      " + darkgreen("Description:") + \
+							"   " + \
+							self.sdict[match].getMetadata("DESCRIPTION") \
+							+ "\n\n")
 				if full_package:
 					try:
-						desc, homepage, license = self.portdb.aux_get(
-							full_package, ["DESCRIPTION","HOMEPAGE","LICENSE"])
+						metadata = dict(zip(metadata_keys,
+							self._aux_get(full_package, metadata_keys)))
 					except KeyError:
 						msg.append("emerge: search: aux_get() failed, skipping\n")
 						continue
+
+					desc = metadata["DESCRIPTION"]
+					homepage = metadata["HOMEPAGE"]
+					license = metadata["LICENSE"]
+
 					if masked:
 						msg.append(green("*") + "  " + \
 							white(match) + " " + red("[ Masked ]") + "\n")
@@ -288,14 +311,19 @@ class search(object):
 					mycat = match.split("/")[0]
 					mypkg = match.split("/")[1]
 					mycpv = match + "-" + myversion
-					myebuild = self.portdb.findname(mycpv)
+					myebuild = self._findname(mycpv)
 					if myebuild:
+						pkg = Package(built=False, cpv=mycpv,
+							installed=False, metadata=metadata,
+							root_config=self.root_config, type_name="ebuild")
 						pkgdir = os.path.dirname(myebuild)
-						from portage import manifest
-						mf = manifest.Manifest(
+						mf = self.settings.repositories.get_repo_for_location(
+							os.path.dirname(os.path.dirname(pkgdir)))
+						mf = mf.load_manifest(
 							pkgdir, self.settings["DISTDIR"])
 						try:
-							uri_map = self.portdb.getFetchMap(mycpv)
+							uri_map = _parse_uri_map(mycpv, metadata,
+								use=pkg.use.enabled)
 						except portage.exception.InvalidDependString as e:
 							file_size_str = "Unknown (%s)" % (e,)
 							del e
@@ -321,12 +349,7 @@ class search(object):
 							break
 
 					if myebuild and file_size_str is None:
-						mystr = str(mysum[0] // 1024)
-						mycount = len(mystr)
-						while (mycount > 3):
-							mycount -= 3
-							mystr = mystr[:mycount] + "," + mystr[mycount:]
-						file_size_str = mystr + " kB"
+						file_size_str = localized_size(mysum[0])
 
 					if self.verbose:
 						if available:
