@@ -125,6 +125,7 @@ SUPPORTED_FEATURES       = frozenset([
 	"buildpkg",
 	"buildsyspkg",
 	"candy",
+	"case-insensitive-fs",
 	"ccache",
 	"cgroup",
 	"chflags",
@@ -189,6 +190,7 @@ SUPPORTED_FEATURES       = frozenset([
 	"unmerge-backup",
 	"unmerge-logs",
 	"unmerge-orphans",
+	"unprivileged",
 	"userfetch",
 	"userpriv",
 	"usersandbox",
