@@ -1,28 +1,34 @@
 # portage: news management code
-# Copyright 2006 Gentoo Foundation
+# Copyright 2006-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: news.py 15541 2010-03-02 19:51:44Z zmedico $
+
+from __future__ import print_function, unicode_literals
 
 __all__ = ["NewsManager", "NewsItem", "DisplayRestriction",
 	"DisplayProfileRestriction", "DisplayKeywordRestriction",
-	"DisplayInstalledRestriction"]
+	"DisplayInstalledRestriction",
+	"count_unread_news", "display_news_notifications"]
 
-import codecs
+import io
 import logging
 import os as _os
 import re
+import portage
+from portage import OrderedDict
 from portage import os
 from portage import _encodings
 from portage import _unicode_decode
 from portage import _unicode_encode
+from portage.const import NEWS_LIB_PATH
 from portage.util import apply_secpass_permissions, ensure_dirs, \
 	grabfile, normalize_path, write_atomic, writemsg_level
 from portage.data import portage_gid
 from portage.dep import isvalidatom
 from portage.localization import _
 from portage.locks import lockfile, unlockfile
-from portage.exception import InvalidLocation, OperationNotPermitted, \
-	PermissionDenied
+from portage.output import colorize
+from portage.exception import (InvalidLocation, OperationNotPermitted,
+	PermissionDenied, ReadOnlyFileSystem)
 
 class NewsManager(object):
 	"""
@@ -40,7 +46,6 @@ class NewsManager(object):
 	def __init__(self, portdb, vardb, news_path, unread_path, language_id='en'):
 		self.news_path = news_path
 		self.unread_path = unread_path
-		self.target_root = vardb.root
 		self.language_id = language_id
 		self.config = vardb.settings
 		self.vdb = vardb
@@ -56,10 +61,12 @@ class NewsManager(object):
 		self._dir_mode  = 0o0074
 		self._mode_mask = 0o0000
 
-		portdir = portdb.porttree_root
-		profiles_base = os.path.join(portdir, 'profiles') + os.path.sep
+		portdir = portdb.repositories.mainRepoLocation()
+		profiles_base = None
+		if portdir is not None:
+			profiles_base = os.path.join(portdir, 'profiles') + os.path.sep
 		profile_path = None
-		if portdb.settings.profile_path:
+		if profiles_base is not None and portdb.settings.profile_path:
 			profile_path = normalize_path(
 				os.path.realpath(portdb.settings.profile_path))
 			if profile_path.startswith(profiles_base):
@@ -115,7 +122,6 @@ class NewsManager(object):
 			except PermissionDenied:
 				return
 
-			updates = []
 			for itemid in news:
 				try:
 					itemid = _unicode_decode(itemid,
@@ -124,7 +130,7 @@ class NewsManager(object):
 					itemid = _unicode_decode(itemid,
 						encoding=_encodings['fs'], errors='replace')
 					writemsg_level(
-						"!!! Invalid encoding in news item name: '%s'\n" % \
+						_("!!! Invalid encoding in news item name: '%s'\n") % \
 						itemid, level=logging.ERROR, noiselevel=-1)
 					continue
 
@@ -174,7 +180,8 @@ class NewsManager(object):
 		unread_lock = None
 		try:
 			unread_lock = lockfile(unread_filename, wantnewlockfile=1)
-		except (InvalidLocation, OperationNotPermitted, PermissionDenied):
+		except (InvalidLocation, OperationNotPermitted, PermissionDenied,
+			ReadOnlyFileSystem):
 			pass
 		try:
 			try:
@@ -238,7 +245,8 @@ class NewsItem(object):
 		for values in self.restrictions.values():
 			any_match = False
 			for restriction in values:
-				if restriction.checkRestriction(**kwargs):
+				if restriction.checkRestriction(
+					**portage._native_kwargs(kwargs)):
 					any_match = True
 			if not any_match:
 				all_match = False
@@ -251,10 +259,11 @@ class NewsItem(object):
 		return self._valid
 
 	def parse(self):
-		lines = codecs.open(_unicode_encode(self.path,
+		f = io.open(_unicode_encode(self.path,
 			encoding=_encodings['fs'], errors='strict'),
-			mode='r', encoding=_encodings['content'], errors='replace'
-			).readlines()
+			mode='r', encoding=_encodings['content'], errors='replace')
+		lines = f.readlines()
+		f.close()
 		self.restrictions = {}
 		invalids = []
 		for i, line in enumerate(lines):
@@ -350,3 +359,67 @@ class DisplayInstalledRestriction(DisplayRestriction):
 		if vdb.match(self.atom):
 			return True
 		return False
+
+def count_unread_news(portdb, vardb, repos=None, update=True):
+	"""
+	Returns a dictionary mapping repos to integer counts of unread news items.
+	By default, this will scan all repos and check for new items that have
+	appeared since the last scan.
+
+	@param portdb: a portage tree database
+	@type portdb: pordbapi
+	@param vardb: an installed package database
+	@type vardb: vardbapi
+	@param repos: names of repos to scan (None means to scan all available repos)
+	@type repos: list or None
+	@param update: check for new items (default is True)
+	@type update: boolean
+	@rtype: dict
+	@return: dictionary mapping repos to integer counts of unread news items
+	"""
+
+	NEWS_PATH = os.path.join("metadata", "news")
+	UNREAD_PATH = os.path.join(vardb.settings['EROOT'], NEWS_LIB_PATH, "news")
+	news_counts = OrderedDict()
+	if repos is None:
+		repos = portdb.getRepositories()
+
+	permission_msgs = set()
+	for repo in repos:
+		try:
+			manager = NewsManager(portdb, vardb, NEWS_PATH, UNREAD_PATH)
+			count = manager.getUnreadItems(repo, update=True)
+		except PermissionDenied as e:
+			# NOTE: The NewsManager typically handles permission errors by
+			# returning silently, so PermissionDenied won't necessarily be
+			# raised even if we do trigger a permission error above.
+			msg = "Permission denied: '%s'\n" % (e,)
+			if msg in permission_msgs:
+				pass
+			else:
+				permission_msgs.add(msg)
+				writemsg_level(msg, level=logging.ERROR, noiselevel=-1)
+			news_counts[repo] = 0
+		else:
+			news_counts[repo] = count
+
+	return news_counts
+
+def display_news_notifications(news_counts):
+	"""
+	Display a notification for unread news items, using a dictionary mapping
+	repos to integer counts, like that returned from count_unread_news().
+	"""
+	newsReaderDisplay = False
+	for repo, count in news_counts.items():
+		if count > 0:
+			if not newsReaderDisplay:
+				newsReaderDisplay = True
+				print()
+			print(colorize("WARN", " * IMPORTANT:"), end=' ')
+			print("%s news items need reading for repository '%s'." % (count, repo))
+
+	if newsReaderDisplay:
+		print(colorize("WARN", " *"), end=' ')
+		print("Use " + colorize("GOOD", "eselect news") + " to read news items.")
+		print()
